<?php

namespace Slovocast\Domain;

use DateTime;
use DateTimeImmutable;

trait Entity
{
    protected ?int $id;
    protected ?DateTimeImmutable $createdAt;
    protected ?DateTime $updatedAt;

    /**
     * Functionally if there is no ID, this is a new Entity that had not been
     * persisted to a datasource.
     *
     * @return bool
     */
    public function isNew(): bool
    {
        return (bool) $this->id;
    }

    public function setId(int $id): static 
    {
        $this->id = $id;
        return $this;
    }

    public function getId(): int
    {
        return $this->id;
    }

    public function setCreatedAt(DateTimeImmutable $createdAt): static 
    {
        $this->createdAt = $createdAt;
        return $this;
    }

    public function getCreatedAt(): DateTimeImmutable
    {
        return $this->createdAt;
    }

    public function setUpdatedAt(DateTime $updatedAt): static 
    {
        $this->updatedAt = $updatedAt;
        return $this;
    }

    public function getUpdatedAt(): DateTime
    {
        return $this->updatedAt;
    }
}
