<?php

namespace Slovocast\Infrastructure;

use Psr\Http\Message\ServerRequestInterface as Request;
use Psr\Http\Message\ResponseInterface as Response;

abstract class Middleware
{
    /**
     * @param Request $request
     * @param Response $response
     * @param callable $next
     * @return Response
     */
    public function __invoke(
        Request $request, 
        Response $response, 
        callable $next
    ): Response {
        return $this->invoke($request, $response, $next);
    }

    /**
     * To write Middleware, we need to implement this concrete method, and set
     * the child class as the Middleware in the Application bootstrapping.
     *
     * @param Request $request
     * @param Response $response
     * @param callable $next
     * @return Response
     */
    abstract public function invoke(
        Request $request,
        Response $response,
        callable $next
    ): Response;
}
